# -*- coding: utf-8 -*-
#
# Kolab Systems, Cyrus IMAP listener for Univention Corporate Server 3.0
#  listener module: creating mailboxes and sieve scripts
#
# Copyright 2004-2011 Univention GmbH
#           2012 Kolab Systems AG
#
# http://www.univention.de/
# http://www.kolabsys.com/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

__package__=''  # workaround for PEP 366
import listener
import os, string, pwd, grp, univention.debug, subprocess

name = 'cyrus'
description = 'Create default IMAP folders for Kolab Groupware users'
filter = '(&(objectClass=kolabInetOrgPerson)(uid=*))'

# TODO: Should come from settings
homeserver_attr = 'univentionMailHomeServer'
result_attr = 'mailPrimaryAddress'

attributes = ['uid', homeserver_attr, result_attr ]

def is_cyrus_murder_backend():
    if (listener.baseConfig.get('mail/cyrus/murder/master') and listener.baseConfig.get('mail/cyrus/murder/backend/hostname')):
        # ucr currently gives '' if not set, might change to None
        return True
    else:
        return False

def create_cyrus_userlogfile(mailaddress):
    userlogfiles = listener.baseConfig.get('mail/cyrus/userlogfiles')

    if userlogfiles and userlogfiles.lower() in ['true', 'yes']:
        path = '/var/lib/cyrus/log/%s' % (mailaddress)

        cyrus_id = pwd.getpwnam('cyrus')[2]
        mail_id = grp.getgrnam('mail')[2]

        if not os.path.exists(path):
            os.mkdir(path)

        os.chmod(path, 0750)
        os.chown(path, cyrus_id, mail_id)

def create_cyrus_mailbox(new):
    if new.has_key(result_attr) and new[result_attr][0]:
        try:
            listener.setuid(0)

            p = os.popen('/usr/sbin/univention-cyrus-mkdir %s' % (string.lower(new[result_attr][0])))
            p.close()

            create_cyrus_userlogfile(string.lower(new[result_attr][0]))
        finally:
            listener.unsetuid()

def delete_cyrus_mailbox(attrs):
    if attrs.has_key(result_attr) and attrs[result_attr][0]:

        try:
            listener.setuid(0)

            p = os.popen('/usr/sbin/univention-cyrus-delete-folder %s' % (string.lower(attrs[result_attr][0])))
            p.close()
        finally:
            listener.unsetuid()

def move_cyrus_murder_mailbox(new, old):
    if new.has_key(result_attr) and new[result_attr][0] and old.has_key(result_attr) and old[result_attr][0]:
        try:
            listener.setuid(0)

            oldemail=string.lower(old[result_attr][0])
            localCyrusMurderBackendFQDN = listener.baseConfig.get('mail/cyrus/murder/backend/hostname')
            if ( localCyrusMurderBackendFQDN.find('.') == -1 ):
                localCyrusMurderBackendFQDN = '%s.%s' % (listener.baseConfig.get('mail/cyrus/murder/backend/hostname'), listener.baseConfig.get('domainname'))

            returncode = subprocess.call("/usr/sbin/univention-cyrus-murder-movemailbox %s %s" % (oldemail, localCyrusMurderBackendFQDN), shell=True)

            if ( returncode != 0 ):
                univention.debug.debug(univention.debug.LISTENER, univention.debug.WARN, '%s: Cyrus Murder mailbox rename failed for %s' % (name, oldemail))

            create_cyrus_userlogfile(string.lower(new[result_attr][0]))
        finally:
            listener.unsetuid()

def handler(dn, new, old):
    """
        DN - always the entry's distinguished name.

        new, old - always dictionaries.

                        | new Empty | new Not Empty
        ----------------+-----------+--------------
        old Empty       | ????????? | add
        old Not Empty   | del       | modify
    """
    fqdn = '%s.%s' % (listener.baseConfig['hostname'], listener.baseConfig['domainname'])

    # Find the univentionMailHomeServer for the user.
    mail_homeserver = None

    if isinstance(new, dict) and len(new.keys()) > 0:
        if new.has_key('univentionMailHomeServer'):
            if isinstance(new[homeserver_attr], list):
                if len(new[homeserver_attr]) == 1:
                    mail_homeserver = new[homeserver_attr][0]
                elif len(new[homeserver_attr]) > 1:
                    # TODO: Log error
                    return
                else:
                    # TODO: Log error
                    return
            else:
                # TODO: Log error
                return
        else:
            # TODO: Log error
            return

        if mail_homeserver == None:
            # Note that with all the if/else's above, we really should not end
            # up here, and the test below is what we actually want.
            # TODO: Log error
            return

        # Not for us.
        if not mail_homeserver == fqdn:
            return

        # If new has keys, and old does not, this is a new mailbox.
        if isinstance(old, dict) and len(old.keys()) < 1:
            create_cyrus_mailbox(new)

        # If old does have keys, this is a modification.
        else:
            old_mail_address = old.get(result_attr, [''])[0]
            old_mail_homeserver = old.get(homeserver_attr, [''])[0]
            mailboxrename_enabled = listener.baseConfig.get('mail/cyrus/mailbox/rename')

            # See if I'm a Cyrus IMAP murder backend.
            if not is_cyrus_murder_backend():
                if not old_mail_address == None and not old_mail_address == '':
                    # Only relevant if mailbox renaming is enabled.
                    if not mailboxrename_enabled == None and mailboxrename_enabled.lower() in ['true', 'yes']:
                        if not old_mail_homeserver == fqdn and mail_homeserver == fqdn:
                            # We are not a murder, and the home server attribute
                            # has changed. The mailbox needs to be created here,
                            # as it cannot be transferred here.
                            create_cyrus_mailbox(new)
                        else:
                            # We are not a murder, but the home server attribute
                            # was NOT changed, and rename is enabled.
                            #
                            # cyrus-mailboxrename.py takes care of this.
                            pass
                    else:
                        # TODO: Log a message that renaming mailboxes is not
                        # enabled.
                        #
                        # Note: The old mailbox is not removed.
                        # Note^2: ACLs referring to the old result attribute
                        #   will inadvertedly be invalidated, but remain active.
                        create_cyrus_mailbox(new)
                else:
                    # There is no old mail address, so this user has just
                    # received a primary mail address.
                    create_cyrus_mailbox(new)
            else:
                if not old_mail_homeserver == None and not old_mail_homeserver == '':
                    move_cyrus_murder_mailbox(new, old)
                else:
                    if old_mail_homeserver == fqdn:
                        # cyrus-mailboxrename.py is supposed to take care of this.
                        return
                    elif mail_homeserver == fqdn:
                        # cyrus-mailboxrename.py on the old mail_homeserver is
                        # supposed to take care of this.
                        #
                        # Note: We should never end up here as we test for this
                        #   earlier.
                        return
                    else:
                        # Neither the target nor the source is us.
                        return

    else: # if isinstance(new, dict) and len(new.keys()) > 0:
        # Deletion.
        mailboxdelete_enabled = listener.baseConfig.get('mail/cyrus/mailbox/delete')
        if not mailboxdelete_enabled == None and mailboxdelete_enabled.lower() in ['', 'true', 'yes']:
            if old.get(homeserver_attr, [''])[0] == fqdn:
                delete_cyrus_mailbox(old)
